@@ -1,6 +1,7 @@
 
 import os
 import sys
+import datetime
 import time
 import traceback
 import six
@@ -16,28 +17,28 @@ from .unittest import TestResult, _TextTestResult, failfast
 # http://stackoverflow.com/questions/1707890/fast-way-to-filter-illegal-xml-unicode-chars-in-python
 
 _illegal_unichrs = [
-    (0x00, 0x08), (0x0B, 0x0C), (0x0E, 0x1F), 
-    (0x7F, 0x84), (0x86, 0x9F), 
+    (0x00, 0x08), (0x0B, 0x0C), (0x0E, 0x1F),
+    (0x7F, 0x84), (0x86, 0x9F),
     (0xFDD0, 0xFDDF), (0xFFFE, 0xFFFF),
-] 
-if sys.maxunicode >= 0x10000:  # not narrow build 
+]
+if sys.maxunicode >= 0x10000:  # not narrow build
     _illegal_unichrs.extend([
-        (0x1FFFE, 0x1FFFF), (0x2FFFE, 0x2FFFF), 
-        (0x3FFFE, 0x3FFFF), (0x4FFFE, 0x4FFFF), 
-        (0x5FFFE, 0x5FFFF), (0x6FFFE, 0x6FFFF), 
-        (0x7FFFE, 0x7FFFF), (0x8FFFE, 0x8FFFF), 
-        (0x9FFFE, 0x9FFFF), (0xAFFFE, 0xAFFFF), 
-        (0xBFFFE, 0xBFFFF), (0xCFFFE, 0xCFFFF), 
-        (0xDFFFE, 0xDFFFF), (0xEFFFE, 0xEFFFF), 
+        (0x1FFFE, 0x1FFFF), (0x2FFFE, 0x2FFFF),
+        (0x3FFFE, 0x3FFFF), (0x4FFFE, 0x4FFFF),
+        (0x5FFFE, 0x5FFFF), (0x6FFFE, 0x6FFFF),
+        (0x7FFFE, 0x7FFFF), (0x8FFFE, 0x8FFFF),
+        (0x9FFFE, 0x9FFFF), (0xAFFFE, 0xAFFFF),
+        (0xBFFFE, 0xBFFFF), (0xCFFFE, 0xCFFFF),
+        (0xDFFFE, 0xDFFFF), (0xEFFFE, 0xEFFFF),
         (0xFFFFE, 0xFFFFF), (0x10FFFE, 0x10FFFF),
-    ]) 
+    ])
 
 _illegal_ranges = [
     "%s-%s" % (six.unichr(low), six.unichr(high))
     for (low, high) in _illegal_unichrs
 ]
 
-INVALID_XML_1_0_UNICODE_RE = re.compile(u'[%s]' % u''.join(_illegal_ranges)) 
+INVALID_XML_1_0_UNICODE_RE = re.compile(u'[%s]' % u''.join(_illegal_ranges))
 
 
 STDOUT_LINE = '\nStdout:\n%s'
@@ -94,7 +95,14 @@ class _TestInfo(object):
         self.test_result = test_result
         self.outcome = outcome
         self.elapsed_time = 0
-        self.err = err
+        self.timestamp = datetime.datetime.min.replace(microsecond=0).isoformat()
+        if err:
+            if self.outcome != _TestInfo.SKIP:
+                self.test_exception_name = safe_unicode(err[0].__name__)
+                self.test_exception_message = safe_unicode(err[1])
+            else:
+                self.test_exception_message = safe_unicode(err)
+
         self.stdout = test_result._stdout_data
         self.stderr = test_result._stderr_data
 
@@ -102,7 +110,7 @@ class _TestInfo(object):
         self.test_exception_info = (
             '' if outcome in (self.SUCCESS, self.SKIP)
             else self.test_result._exc_info_to_string(
-                    self.err, test_method)
+                    err, test_method)
         )
 
         self.test_name = testcase_name(test_method)
@@ -118,6 +126,8 @@ class _TestInfo(object):
         """
         self.elapsed_time = \
             self.test_result.stop_time - self.test_result.start_time
+        timestamp = datetime.datetime.fromtimestamp(self.test_result.stop_time)
+        self.timestamp = timestamp.replace(microsecond=0).isoformat()
 
     def get_description(self):
         """
@@ -337,6 +347,10 @@ class _XMLTestResult(_TextTestResult):
         testsuite.setAttribute(
             'time', '%.3f' % sum(map(lambda e: e.elapsed_time, tests))
         )
+        if tests:
+            testsuite.setAttribute(
+                'timestamp', max(map(lambda e: e.timestamp, tests))
+            )
         failures = filter(lambda e: e.outcome == e.FAILURE, tests)
         testsuite.setAttribute('failures', str(len(list(failures))))
 
@@ -345,7 +359,7 @@ class _XMLTestResult(_TextTestResult):
 
         skips = filter(lambda e: e.outcome == _TestInfo.SKIP, tests)
         testsuite.setAttribute('skipped', str(len(list(skips))))
- 
+
         _XMLTestResult._report_testsuite_properties(
             testsuite, xml_document, properties)
 
@@ -414,6 +428,7 @@ class _XMLTestResult(_TextTestResult):
             'name', _XMLTestResult._test_method_name(test_result.test_id)
         )
         testcase.setAttribute('time', '%.3f' % test_result.elapsed_time)
+        testcase.setAttribute('timestamp', test_result.timestamp)
 
         if (test_result.outcome != test_result.SUCCESS):
             elem_name = ('failure', 'error', 'skipped')[test_result.outcome-1]
@@ -422,18 +437,18 @@ class _XMLTestResult(_TextTestResult):
             if test_result.outcome != test_result.SKIP:
                 failure.setAttribute(
                     'type',
-                    safe_unicode(test_result.err[0].__name__)
+                    test_result.test_exception_name
                 )
                 failure.setAttribute(
                     'message',
-                    safe_unicode(test_result.err[1])
+                    test_result.test_exception_message
                 )
                 error_info = safe_unicode(test_result.get_error_info())
                 _XMLTestResult._createCDATAsections(
                     xml_document, failure, error_info)
             else:
                 failure.setAttribute('type', 'skip')
-                failure.setAttribute('message', safe_unicode(test_result.err))
+                failure.setAttribute('message', test_result.test_exception_message)
 
     _report_testcase = staticmethod(_report_testcase)
 
